#include "motor.h"
#include "zerobtn.h"
#include "gapchecker.h"
#include "DFRobot_SpeechSynthesis.h"

#define MEDICINE_TYPES 3

DFRobot_SpeechSynthesis_I2C ss;

struct MedicineMsg {
  String msg;  // eg:请吃msg药
  int num;     // eg:请吃num颗
  int boxNum;  // 药所在的格子 1~7
  int startTime; // 开始执行吃药的时间
  bool eaten; // 这颗药是不是已经吃了
};

MedicineMsg medicines[MEDICINE_TYPES] = {
  // 时间需要从小到大排列，且两两间隔不能小于20s
  {msg : "感冒药",   num : 2,  boxNum : 1,  startTime : 3, eaten : false},
  {msg : "头孢",     num : 3,  boxNum : 4,  startTime : 18, eaten : false},
  {msg : "阿司匹林", num : 7,  boxNum : 7,  startTime : 38, eaten : false},
};

int i_tar;  // 转动药盒时对应的药品信息
int passedCells;  // 转动药盒时已经转过的药盒

String curState("none");
float initTime; // 初始化所用的时间

void voiceRemind(String msg, int num){
  String totalMsg = (String)"请吃" + num + (String)"颗" + msg;
  Serial.println(totalMsg);
  ss.speak(totalMsg);
}

// 归零成功返回true，否则返回false
bool ZeroDevice(){
  // 归零模式
  if(IsZerobtnClicked()){
    // 当限位器被触动时，回位一定距离
    StartMotor();
    delay(20);
    StopMotor();
    return true;
  }
  else{
    // 当限位器没有被按时，反转电机
    ReverseMotor();
    return false;
  }
}

void initSS(){
  //初始化语音合成传感器
  ss.begin();
  //设置语音的音量大小为5
  ss.setVolume(10);
  //设置语音的播放速度为5
  //ss.setSpeed(5);
  //设置发音人为女性
  ss.setSoundType(ss.FEMALE3);
  //设置音调为5
  //ss.setTone(5);
  //设置英文以单词发音
  //ss.setEnglishPron(ss.WORD);
}

void setup() {
  Serial.begin(9600);
  InitMotor();
  initSS();
  curState = "ZeroDevice";
  passedCells = 0;
  initTime = 0;
  ss.speak("欢迎使用智能药盒 我是你的健康助理 小亮 你的健康由我来守护");
  delay(500);
}

void loop() {
  if(curState == "ZeroDevice"){
    bool zeroEnded = ZeroDevice();
    if(zeroEnded){
      // 切换到下一个状态
      curState = "WaitMedicineState";
      // 第一次归零记录下归零所用的时间
      if(initTime == 0)
        initTime = millis()/1000.0;
    }
  }
  else if(curState == "WaitMedicineState"){
    int eatens = 0;
    for(int i = 0; i < MEDICINE_TYPES; i++){
      if(medicines[i].eaten){
        eatens++;
        continue;
      }
      if(millis()/1000.0 - initTime >= medicines[i].startTime){
        if(medicines[i].boxNum >= 1 && medicines[i].boxNum <= 7){
          Serial.println((String)"开始吃" + medicines[i].msg);
          // 当需要吃药时，进入转动药盒状态
          curState = "RotateBoxState";
          i_tar = i;  // 记录下当前药品的序号
          InitChecker(medicines[i].boxNum);
          ss.speak("亲爱的 现在需要吃" + medicines[i].msg + "了");
          //delay(1000);
          StartMotor();
        }
        else{
          Serial.println((String)"boxNum需要在1~7之间 i:" + i + " boxNum:" + medicines[i].boxNum);
          medicines[i].eaten = true;
        }
      }
    }
    //Serial.println((String)"eatens:"+eatens+"/"+MEDICINE_TYPES);
    if(eatens >= MEDICINE_TYPES){
      curState = "EndTip";
    }
  }
  else if(curState == "RotateBoxState"){
    if(IsTargetReached()){
      // 当旋转到目标格子后，回正一定时间
      ReverseMotor();
      delay(20);
      // 停止马达
      StopMotor();
      delay(500);
      voiceRemind(medicines[i_tar].msg, medicines[i_tar].num);
      medicines[i_tar].eaten = true;
      delay(5000); // 等待7s，让用户吃药
      ss.speak("药已吃完");
      delay(1000);
      // 切换状态
      curState = "ZeroDevice";
    }
  }
  else if(curState == "EndTip"){
    Serial.println("EndTip");
    delay(1000);
    ss.speak("亲爱的 您已经吃完了本疗程 祝您早日康复 小亮会一直守护您的健康");
    curState = "EndIdle";
  }
  else if(curState == "EndIdle"){
    Serial.println("EndIdle");
    delay(10000);
  }
}
